<?php

namespace Drupal\digital_signage_framework;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Entity\ContentEntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Provides services around entity types.
 */
class EntityTypes {

  public const EXCLUDED = [
    'aggregator_feed',
    'aggregator_item',
    'block_content',
    'comment',
    'commerce_product_variation_type',
    'contact_message',
    'content_moderation_state',
    'crop',
    'danse_event',
    'danse_notification',
    'danse_notification_action',
    'digital_signage_content_setting',
    'digital_signage_device',
    'digital_signage_schedule',
    'entity_embed_fake_entity',
    'helpdesk_issue',
    'menu_link_content',
    'onesignal_device',
    'paragraph',
    'paragraphs_library_item',
    'path_alias',
    'redirect',
    'shortcut',
    'smart_date_override',
    'smart_date_rule',
    'wayfinding',
    'webform_submission',
    'workspace',
  ];


  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected ConfigFactory $configFactory;

  /**
   * Renderer constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The config factory.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactory $config_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * Gets all entity type definitions.
   *
   * @return \Drupal\Core\Entity\EntityTypeInterface[]
   *   The list of all entity type definitions.
   */
  public function all(): array {
    return $this->entityTypeManager->getDefinitions();
  }

  /**
   * Gets all entity type IDs.
   *
   * @return string[]
   *   All entity type IDs.
   */
  public function allIds(): array {
    return array_keys($this->all());
  }

  /**
   * Gets all entity type IDs that are not ignored.
   *
   * @return string[]
   *   All entity type IDs that are not ignored.
   */
  public function allEnabledIds(): array {
    return array_diff($this->allIds(), $this->allDisabledIds(), $this->excludedIds());
  }

  /**
   * Indicates whether the specified entity type is enabled.
   *
   * "enabled" means it is neither excluded nor disabled through config.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return bool
   *   TRUE if the entity type is enabled, FALSE otherwise.
   */
  public function isEnabled(string $entityTypeId): bool {
    return in_array($entityTypeId, $this->allEnabledIds(), TRUE);
  }

  /**
   * Indicates whether the specified entity type is disabled.
   *
   * "disabled" means it has been deactivated through config. The entity type
   * will still have base fields added, but no special view modes.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return bool
   *   TRUE if the entity type is disabled, FALSE otherwise.
   */
  public function isDisabled(string $entityTypeId): bool {
    return in_array($entityTypeId, $this->allDisabledIds(), TRUE);
  }

  /**
   * Indicates whether the specified entity type is excluded.
   *
   * "excluded" means it is contained in the hard-coded list of entity types
   * that won't receive any special treatment from this module.
   *
   * @param string $entityTypeId
   *   The entity type ID.
   *
   * @return bool
   *   TRUE if the entity type is excluded, FALSE otherwise.
   */
  public function isExcluded(string $entityTypeId): bool {
    return in_array($entityTypeId, $this->excludedIds(), TRUE);
  }

  /**
   * Gets all available entity types.
   *
   * Collects a list of all content entity types that are not excluded and
   * available for disabling through config.
   *
   * @return string[]
   *   A list of entity type ID => entity type label tuples
   */
  public function getAvailableContentEntityTypes(): array {
    $definitions = [];
    foreach ($this->all() as $definition) {
      if (!in_array($definition->id(), $this->excludedIds()) && $definition instanceof ContentEntityTypeInterface) {
        $definitions[$definition->id()] = $definition->getLabel();
      }
    }

    return $definitions;
  }

  /**
   * Get all entity type IDs disabled through config.
   *
   * @return string[]
   *   All entity type IDs that are disabled.
   */
  public function allDisabledIds(): array {
    return $this->configFactory
      ->get('digital_signage_framework.settings')
      ->get('disabled_entity_types') ?: [];
  }

  /**
   * Get all excluded entity type IDs.
   *
   * @return string[]
   *   All entity type IDs that are excluded.
   */
  public function excludedIds(): array {
    // @todo Allow other modules to alter this list.
    return self::EXCLUDED;
  }

}
