<?php

namespace Drupal\digital_signage_framework\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\digital_signage_framework\EntityTypes;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Digital Signage settings for this site.
 */
class FrameworkSettings extends ConfigFormBase {

  /**
   * The entity types service.
   *
   * @var \Drupal\digital_signage_framework\EntityTypes
   */
  protected EntityTypes $entityTypesService;

  /**
   * Constructs the form for the module settings.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\digital_signage_framework\EntityTypes $entityTypesService
   *   The entity types service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypes $entityTypesService) {
    parent::__construct($config_factory);
    $this->entityTypesService = $entityTypesService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): FrameworkSettings {
    return new FrameworkSettings(
      $container->get('config.factory'),
      $container->get('digital_signage_content_setting.entity_types')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'digital_signage_framework_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['digital_signage_framework.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['cron_sync_devices'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Sync devices during cron'),
      '#default_value' => $this->config('digital_signage_framework.settings')->get('cron_sync_devices'),
    ];

    $form['cron_create_schedules'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Create schedules during cron'),
      '#default_value' => $this->config('digital_signage_framework.settings')->get('cron_create_schedules'),
    ];

    $form['hotfix_svg'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Apply hotfix for SVG files'),
      '#default_value' => $this->config('digital_signage_framework.settings')->get('hotfix_svg'),
      '#description' => $this->t('Some devices can not display SVG files with missing <?xml> prefix in them. This option allows to dynamically fix that.'),
    ];

    $form['plantuml_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('PlantUML server'),
      '#default_value' => $this->config('digital_signage_framework.settings')->get('plantuml_url'),
    ];

    $form['css'] = [
      '#type' => 'textarea',
      '#title' => $this->t('CSS files'),
      '#default_value' => $this->config('digital_signage_framework.settings')->get('css'),
    ];

    $form['http_header'] = [
      '#type' => 'textarea',
      '#title' => $this->t('HTTP request header'),
      '#default_value' => $this->config('digital_signage_framework.settings')->get('http_header'),
      '#attributes' => ['data-yaml-editor' => 'true'],
    ];

    $availableContentEntityTypes = $this->entityTypesService->getAvailableContentEntityTypes();
    uasort($availableContentEntityTypes, function ($a, $b) {
      return (string) $a <=> (string) $b;
    });
    $form['disabled_entity_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Disabled entity types'),
      '#description' => $this->t('Select entity types for which you want to fully disable the Digital Signage functionality.'),
      '#default_value' => $this->entityTypesService->allDisabledIds(),
      '#multiple' => TRUE,
      '#options' => $availableContentEntityTypes,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $disabled_entity_types = array_keys(array_filter($form_state->getValue('disabled_entity_types')));
    sort($disabled_entity_types);
    $this->config('digital_signage_framework.settings')
      ->set('cron_sync_devices', $form_state->getValue('cron_sync_devices'))
      ->set('cron_create_schedules', $form_state->getValue('cron_create_schedules'))
      ->set('hotfix_svg', $form_state->getValue('hotfix_svg'))
      ->set('plantuml_url', $form_state->getValue('plantuml_url'))
      ->set('css', $form_state->getValue('css'))
      ->set('http_header', $form_state->getValue('http_header'))
      ->set('disabled_entity_types', $disabled_entity_types)
      ->save();
    parent::submitForm($form, $form_state);
  }

}
